; Smooths an array with a boxcar average of a specified width.
;
; result = SMOOTH_a(array, width)
;
;   array  : The array to be smoothed. 
;            (In PV-WAVE, The array can be of any number of dimensions.)
;   width  : The width of the smoothing window. 
;            Should be an odd number, smaller than the smallest dimension. (If width is even, width + 1 is used instead.)
;
;   result : A copy of array smoothed with a boxcar average of the specified width. It has the same type and dimensions as array.
;
; Note:
;    Result = SMOOTH( Array, Width [, /EDGE_TRUNCATE] [, MISSING=value] [,/NAN] 
;    Array : The array to be smoothed. Array can have any number of dimensions.
;    Width : The width of the smoothing window.
;            Width can either be a scalar or a vector with length equal to the
;           number of dimensions of Array. If Width is a scalar then the same 
;           width is applied for each dimension that has length greater than 1
;           (dimensions of length 1 are skipped). If Width is a vector, then
;           each element of Width is used to specify the smoothing width for
;           each dimension of Array. Values for Width must be smaller than the
;           corresponding Array dimension. If a Width value is even, then 
;           Width+1 will be used instead. The value of Width does not affect
;           the running time of SMOOTH to a great extent.
;
; Some of the results have different values compared to PV-WAVE.
; 	GDL    : 7.76511e-05(0.0000776511) Float
; 	PV-WAVE: 7.76512e-05(0.0000776512) Float
;
; Copyright 2022 National Institute for Fusion Science, LABCOM
; PV-WAVE is a registered trademark of Perforce Software, Inc.
;
FUNCTION smooth_a,array,width
compile_opt idl2, hidden
	aif  = size(array)
        dims = aif(0)
        nelm = aif(dims+2)
        last = aif(dims)

	FOR i=1,dims DO BEGIN
 		IF aif(i) NE 1 THEN last = aif(i)
	ENDFOR
	IF last NE nelm THEN BEGIN
		Message,'It does not support arrays with elements in multiple dimensions.'
	ENDIF
	IF dims GT 5 THEN BEGIN 
		Message,'The number of dimensions of the array is large and it is not supported.'
	ENDIF

	IF dims EQ 1 THEN return,smooth(array,width)
	
	tmp = reform(array,nelm)
	tmp_sm = smooth(tmp,width) 
	CASE dims OF
		2 : tmp_sm = reform(tmp_sm, aif(1), aif(2), /OVERWRITE)
		3 : tmp_sm = reform(tmp_sm, aif(1), aif(2), aif(3), /OVERWRITE)
		4 : tmp_sm = reform(tmp_sm, aif(1), aif(2), aif(3), aif(4), /OVERWRITE)
		5 : tmp_sm = reform(tmp_sm, aif(1), aif(2), aif(3), aif(4), aif(5), /OVERWRITE)
		ELSE: Message,'The number of dimensions of the array is large and it is not supported.'
	ENDCASE 
	return, tmp_sm 
END
